@@ -1,7 +1,7 @@
 /*
  * libSpiff - XSPF playlist handling library
  *
- * Copyright (C) 2007, Sebastian Pipping / Xiph.Org Foundation
+ * Copyright (C) 2006-2008, Sebastian Pipping / Xiph.Org Foundation
  * All rights reserved.
  *
  * Redistribution  and use in source and binary forms, with or without
@@ -39,14 +39,41 @@
  */
 
 #include <spiff/Spiff.h>
-#include <stdio.h>
-#include <cstdlib>
+#include <uriparser/Uri.h>
+#include <cstdio>
+#include <cstdlib> // MAX_PATH
+#include <climits> // PATH_MAX
+
+
+#if defined(__WIN32__) || defined(WIN32)
+# define DIR_SEPERATOR '\\'
+# define PORT_PATH_MAX MAX_PATH
+# if defined(UNICODE)
+#  define uriFilenameToUriString uriWindowsFilenameToUriStringW
+# define PORT_GETCWD _wgetcwd
+# else
+#  define uriFilenameToUriString uriWindowsFilenameToUriStringA
+#  define PORT_GETCWD _getcwd
+# endif
+#else
+# define PORT_PATH_MAX PATH_MAX
+# define DIR_SEPERATOR '/'
+# define uriFilenameToUriString uriUnixFilenameToUriStringA
+# define PORT_GETCWD getcwd
+#endif
+
 
 using namespace Spiff;
 
 
+class DemoReaderCallback : public SpiffReaderCallback {
 
-class DemoPlaylist : public SpiffReaderCallback {
+	bool valid;
+
+public:
+	DemoReaderCallback() : valid(true) {
+
+	}
 
 private:
 	void addTrack(SpiffTrack * track) {
@@ -55,12 +82,12 @@ private:
 			printKeyValue(_PT("Annotation"), track->getAnnotation());
 			printKeyValue(_PT("Creator"), track->getCreator());
 			printKeyValue(_PT("Duration"), track->getDuration());
-			printIdentifiers(track);
+			printIdentifierSummary(track);
 			printKeyValue(_PT("Image"), track->getImage());
 			printKeyValue(_PT("Info"), track->getInfo());
-			printLinks<SpiffTrack>(track);
-			printLocations(track);
-			printMetas<SpiffTrack>(track);
+			printLinkSummay<SpiffTrack>(track);
+			printLocationSummary(track);
+			printMetaSummary<SpiffTrack>(track);
 			printKeyValue(_PT("Title"), track->getTitle());
 			printKeyValue(_PT("TrackNum"), track->getTrackNum());
 		close();
@@ -70,23 +97,38 @@ private:
 	void setProps(SpiffProps * props) {
 		open(_PT("Props"));
 			printKeyValue(_PT("Annotation"), props->getAnnotation());
-			printAttributions(props);
+			printAttributionSummary(props);
 			printKeyValue(_PT("Creator"), props->getCreator());
 			printKeyValue(_PT("Date"), props->getDate());
+
 			printKeyValue(_PT("Identifier"), props->getIdentifier());
 			printKeyValue(_PT("Image"), props->getImage());
 			printKeyValue(_PT("Info"), props->getInfo());
 			printKeyValue(_PT("License"), props->getLicense());
-			printLinks<SpiffProps>(props);
+			printLinkSummay<SpiffProps>(props);
 			printKeyValue(_PT("Location"), props->getLocation());
-			printMetas<SpiffProps>(props);
+			printMetaSummary<SpiffProps>(props);
 			printKeyValue(_PT("Title"), props->getTitle());
 			printKeyValue(_PT("Version"), props->getVersion());
 		close();
 		delete props;
 	}
 
-	static void open(const XML_Char * type) {
+	bool handleError(int line, int column, int errorCode,
+			XML_Char const * description) {
+		PORT_PRINTF(_PT("Error %i at (line %i, column %i): '%s'\n"),
+				errorCode, line, column, description);
+		valid = false;
+		return true; // Continue parsing
+	}
+
+	void notifySuccess() {
+		if (valid) {
+			PORT_PRINTF(_PT("Everything fine.\n"));
+		}
+	}
+
+	static void open(XML_Char const * type) {
 		PORT_PRINTF(_PT("%s\n"), type);
 	}
 
@@ -94,7 +136,7 @@ private:
 		PORT_PRINTF(_PT("\n"));
 	}
 
-	static void printKeyValue(const XML_Char * key, const XML_Char * value) {
+	static void printKeyValue(XML_Char const * key, XML_Char const * value) {
 		if (value == NULL) {
 			PORT_PRINTF(_PT("\t%s: -\n"), key);
 		} else {
@@ -102,7 +144,7 @@ private:
 		}
 	}
 
-	static void printKeyValue(const XML_Char * key, int value) {
+	static void printKeyValue(XML_Char const * key, int value) {
 		if (value == -1) {
 			PORT_PRINTF(_PT("\t%s: -\n"), key);
 		} else {
@@ -110,7 +152,7 @@ private:
 		}
 	}
 
-	static void printKeyValue(const XML_Char * key, const SpiffDateTime * value) {
+	static void printKeyValue(XML_Char const * key, SpiffDateTime const * value) {
 		if (value == NULL) {
 			PORT_PRINTF(_PT("\t%s: -\n"), key);
 		} else {
@@ -121,48 +163,106 @@ private:
 		}
 	}
 
-	static void printHelper(const XML_Char * key, int size) {
+	static void printHelper(XML_Char const * key, int size) {
 		PORT_PRINTF(_PT("\t%s: %i entr%s\n"), key, size, (size == 1) ? _PT("y") : _PT("ies"));
 	}
 
 	template <class T>
-	static void printLinks(T * data) {
-		const int size = (static_cast<SpiffData *>(data))->getLinkCount();
+	static void printLinkSummay(T * data) {
+		int const size = (static_cast<SpiffData *>(data))->getLinkCount();
 		printHelper(_PT("Links"), size);
 	}
 
 	template <class T>
-	static void printMetas(T * data) {
-		const int size = (static_cast<SpiffData *>(data))->getMetaCount();
+	static void printMetaSummary(T * data) {
+		int const size = (static_cast<SpiffData *>(data))->getMetaCount();
 		printHelper(_PT("Metas"), size);
 	}
 
-	static void printAttributions(SpiffProps * props) {
-		const int size = props->getAttributionCount();
+	static void printAttributionSummary(SpiffProps * props) {
+		int const size = props->getAttributionCount();
 		printHelper(_PT("Attributions"), size);
 	}
 
-	static void printIdentifiers(SpiffTrack * track) {
-		const int size = track->getIdentifierCount();
+	static void printIdentifierSummary(SpiffTrack * track) {
+		int const size = track->getIdentifierCount();
 		printHelper(_PT("Identifier"), size);
 	}
 
-	static void printLocations(SpiffTrack * track) {
-		const int size = track->getLocationCount();
+	static void printLocationSummary(SpiffTrack * track) {
+		int const size = track->getLocationCount();
 		printHelper(_PT("Locations"), size);
 	}
 
 };
 
 
-
-int PORT_MAIN() {
-	SpiffReader reader;
-	const int res = reader.parseFile(_PT("playlist.xspf"), new DemoPlaylist());
-	if (res != SPIFF_READER_SUCCESS) {
-		PORT_PRINTF(_PT("Error %i at line %i: '%s'\n"), res, reader.getErrorLine(), reader.getErrorText());
-	} else {
-		PORT_PRINTF(_PT("Everything fine.\n"));
+/**
+ * Creates a URI from an absolute filename for this host sytem.
+ * 
+ * @param fileName  Absolute filename
+ * @return          New'ed Absolute URI or NULL on error
+ */
+XML_Char *
+makeUriFromAbsoluteFilename(XML_Char const * fileName) {
+	if (fileName == NULL) {
+		return NULL;
 	}
-	return 0;
+
+	size_t const len = ::PORT_STRLEN(fileName);
+	size_t const charsNeeded = 8 + 3 * len + 1;
+	XML_Char * const absUri = new XML_Char[charsNeeded];
+	::uriFilenameToUriString(fileName, absUri);
+	return absUri;
+}
+
+
+/**
+ * Prepends the current working directory and a (back)slash
+ * (depending on the host operating system) to the relative filename given.
+ *
+ * @param relativeFilename  Relative filename of host operating system
+ * @return                  Absolute filename of host operating system
+ */
+XML_Char *
+makeAbsoluteFilename(XML_Char const * relativeFilename) {
+	if (relativeFilename == NULL) {
+		return NULL;
+	}
+	size_t const relativeFilenameLen = ::PORT_STRLEN(relativeFilename);
+	XML_Char * const absoluteFilename = new XML_Char[
+			PORT_PATH_MAX + 1 + relativeFilenameLen + 1];
+	XML_Char const * const res = ::PORT_GETCWD(absoluteFilename,
+			PORT_PATH_MAX);
+	if (res == NULL) {
+		delete [] absoluteFilename;
+		return NULL;
+	}
+	size_t const cwdLen = ::PORT_STRLEN(absoluteFilename);
+	absoluteFilename[cwdLen] = _PT(DIR_SEPERATOR);
+	::PORT_STRNCPY(absoluteFilename + cwdLen + 1, relativeFilename,
+			relativeFilenameLen);
+	absoluteFilename[cwdLen + 1 + relativeFilenameLen] = _PT('\0');
+	return absoluteFilename;
+}
+
+
+int
+PORT_MAIN() {
+	SpiffReader reader;
+	SpiffReaderCallback * callback = new DemoReaderCallback;
+
+	// Make base URI
+	XML_Char const * const relativeFilename = _PT("playlist.xspf");
+	XML_Char * const absoluteFilename = makeAbsoluteFilename(relativeFilename);
+	XML_Char * const baseUri = makeUriFromAbsoluteFilename(absoluteFilename);
+	delete [] absoluteFilename;
+
+	// Parse
+	int const res = reader.parseFile(relativeFilename,
+			callback, baseUri);
+	delete [] baseUri;
+	delete callback;
+
+	return (res == SPIFF_READER_SUCCESS) ? 0 : 1;
 }
