@@ -1,7 +1,7 @@
 /*
  * libSpiff - XSPF playlist handling library
  *
- * Copyright (C) 2007, Sebastian Pipping / Xiph.Org Foundation
+ * Copyright (C) 2006-2008, Sebastian Pipping / Xiph.Org Foundation
  * All rights reserved.
  *
  * Redistribution  and use in source and binary forms, with or without
@@ -47,14 +47,20 @@
 #include <spiff/ProjectOpus/ProjectOpusPlaylistExtension.h>
 #include <spiff/SpiffReader.h>
 #include <spiff/SpiffStack.h>
-#include <stdio.h>
+#include <spiff/SpiffToolbox.h>
+#include <cstdio>
 #include <cstring>
 
+
+/// @cond DOXYGEN_INGORE
+#define PROJECT_OPUS_TEXT_ZERO_ATTRIBUTE_OUT_OF_SET(attr, set) _PT("Attribute '") attr _PT("' must be in ") set _PT(".")
+/// @endcond
+
+
 namespace Spiff {
 namespace ProjectOpus {
 
 
-
 /**
  * Holds tag types that are pushed on the readers's tag stack.
  */
@@ -63,7 +69,6 @@ enum ProjectOpusTag {
 };
 
 
-
 /// @cond DOXYGEN_NON_API
 
 /**
@@ -96,7 +101,6 @@ class ProjectOpusPlaylistExtensionReaderPrivate {
 /// @endcond
 
 
-
 ProjectOpusPlaylistExtensionReader::ProjectOpusPlaylistExtensionReader(
 		SpiffReader * reader)
 		: SpiffExtensionReader(reader),
@@ -105,16 +109,14 @@ ProjectOpusPlaylistExtensionReader::ProjectOpusPlaylistExtensionReader(
 }
 
 
-
-ProjectOpusPlaylistExtensionReader::ProjectOpusPlaylistExtensionReader(const ProjectOpusPlaylistExtensionReader & source)
+ProjectOpusPlaylistExtensionReader::ProjectOpusPlaylistExtensionReader(ProjectOpusPlaylistExtensionReader const & source)
 		: SpiffExtensionReader(source),
 		d(new ProjectOpusPlaylistExtensionReaderPrivate(*(source.d))) {
 
 }
 
 
-
-ProjectOpusPlaylistExtensionReader & ProjectOpusPlaylistExtensionReader::operator=(const ProjectOpusPlaylistExtensionReader & source) {
+ProjectOpusPlaylistExtensionReader & ProjectOpusPlaylistExtensionReader::operator=(ProjectOpusPlaylistExtensionReader const & source) {
 	if (this != &source) {
 		SpiffExtensionReader::operator=(source);
 		*(this->d) = *(source.d);
@@ -123,27 +125,25 @@ ProjectOpusPlaylistExtensionReader & ProjectOpusPlaylistExtensionReader::operato
 }
 
 
-
 ProjectOpusPlaylistExtensionReader::~ProjectOpusPlaylistExtensionReader() {
 	delete this->d;
 }
 
 
-
-bool ProjectOpusPlaylistExtensionReader::handleExtensionStart(const XML_Char * fullName,
-		const XML_Char ** atts) {
-	switch (this->stack->getSize() + 1) {
+bool ProjectOpusPlaylistExtensionReader::handleExtensionStart(XML_Char const * fullName,
+		XML_Char const ** atts) {
+	switch (this->getElementStack().size() + 1) {
 	case 2:
 		// We only get called on this level for playlist.extension
 		// Name and attributes have already been checked.
-		this->stack->push(TAG_PLAYLIST_EXTENSION);
+		this->getElementStack().push(TAG_PLAYLIST_EXTENSION);
 		return true;
 
 	case 3:
 		// Check and skip namespace
 		if (::PORT_STRNCMP(fullName, PROJECT_OPUS_NS_HOME, PROJECT_OPUS_NS_HOME_LEN)
 				|| ::PORT_STRCMP(fullName + PROJECT_OPUS_NS_HOME_LEN + 1, _PT("info"))) {
-			setError(SPIFF_READER_ERROR_ELEMENT_FORBIDDEN,
+			handleError(SPIFF_READER_ERROR_ELEMENT_FORBIDDEN,
 					SPIFF_READER_TEXT_ONE_ELEMENT_FORBIDDEN,
 					fullName);
 			return false;
@@ -151,7 +151,7 @@ bool ProjectOpusPlaylistExtensionReader::handleExtensionStart(const XML_Char * f
 
 		// One info at most
 		if (!this->d->firstInfo) {
-			setError(SPIFF_READER_ERROR_ELEMENT_TOOMANY,
+			handleError(SPIFF_READER_ERROR_ELEMENT_TOOMANY,
 					SPIFF_READER_TEXT_ZERO_TOO_MANY_ELEMENTS(PROJECT_OPUS_NS_HOME, _PT("info")));
 			return false;
 		}
@@ -161,31 +161,30 @@ bool ProjectOpusPlaylistExtensionReader::handleExtensionStart(const XML_Char * f
 		}
 
 		this->d->firstInfo = false;
-		this->stack->push(TAG_OPUS_PROJECT_INFO);
+		this->getElementStack().push(TAG_OPUS_PROJECT_INFO);
 		return true;
 
 	case 4:
-		if (this->stack->top() == TAG_PLAYLIST_TRACKLIST_TRACK) {
+		if (this->getElementStack().top() == TAG_PLAYLIST_TRACKLIST_TRACK) {
 			// Name and attributes have already been checked.
-			this->stack->push(TAG_PLAYLIST_TRACKLIST_TRACK_EXTENSION);
+			this->getElementStack().push(TAG_PLAYLIST_TRACKLIST_TRACK_EXTENSION);
 			return true;
 		}
 		break;
 
 	}
 
-	this->stack->push(TAG_UNKNOWN);
+	this->getElementStack().push(TAG_UNKNOWN);
 	return true;
 }
 
 
-
-bool ProjectOpusPlaylistExtensionReader::handleExtensionEnd(const XML_Char * /*fullName*/) {
-	switch (this->stack->getSize()) {
+bool ProjectOpusPlaylistExtensionReader::handleExtensionEnd(XML_Char const * /*fullName*/) {
+	switch (this->getElementStack().size()) {
 	case 2:
 		// One info minimum
 		if (this->d->firstInfo) {
-			setError(SPIFF_READER_ERROR_ELEMENT_MISSING,
+			handleError(SPIFF_READER_ERROR_ELEMENT_MISSING,
 					SPIFF_READER_TEXT_ZERO_ELEMENT_MISSING(PROJECT_OPUS_NS_HOME, _PT("info")));
 			return false;
 		}
@@ -193,73 +192,74 @@ bool ProjectOpusPlaylistExtensionReader::handleExtensionEnd(const XML_Char * /*f
 
 	}
 
-	this->stack->pop();
+	this->getElementStack().pop();
 	return true;
 }
 
 
-
-bool ProjectOpusPlaylistExtensionReader::handleExtensionCharacters(const XML_Char * /*s*/, int /*len*/) {
+bool ProjectOpusPlaylistExtensionReader::handleExtensionCharacters(XML_Char const * /*s*/, int /*len*/) {
 	return true;
 }
 
 
-
 SpiffExtension * ProjectOpusPlaylistExtensionReader::wrap() {
 	return this->d->extension.clone();
 }
 
 
-
 SpiffExtensionReader * ProjectOpusPlaylistExtensionReader::createBrother(SpiffReader * reader) const {
 	return new ProjectOpusPlaylistExtensionReader(reader);
 }
 
 
-
-bool ProjectOpusPlaylistExtensionReader::handleInfoAttribs(const XML_Char ** atts) {
+bool ProjectOpusPlaylistExtensionReader::handleInfoAttribs(XML_Char const ** atts) {
 	bool typeFound = false;
 	bool nidFound = false;
 
-	while (atts[0] != NULL) {
-		if (!::PORT_STRCMP(atts[0], _PT("type"))) {
+	for (int i = 0; atts[i] != NULL; i += 2) {
+		if (!::PORT_STRCMP(atts[i], _PT("type"))) {
 			ProjectOpusPlaylistType dummyType = TYPE_PLAYLIST; // Init not needed
-			if (!::PORT_STRCMP(atts[1], _PT("album"))) {
+			if (!::PORT_STRCMP(atts[i], _PT("album"))) {
 				dummyType = TYPE_ALBUM;
-			} else if (!::PORT_STRCMP(atts[1], _PT("playlist"))) {
+			} else if (!::PORT_STRCMP(atts[i + 1], _PT("playlist"))) {
 				dummyType = TYPE_PLAYLIST;
 			} else {
-				setError(SPIFF_READER_ERROR_ATTRIBUTE_INVALID,
+				handleError(SPIFF_READER_ERROR_ATTRIBUTE_INVALID,
 						PROJECT_OPUS_TEXT_ZERO_ATTRIBUTE_OUT_OF_SET(_PT("type"), _PT("{'album', 'playlist'}")));
 				return false;
 			}
 			this->d->extension.setType(dummyType);
 			typeFound = true;
-		} else if (!::PORT_STRCMP(atts[0], _PT("nid"))) {
+		} else if (!::PORT_STRCMP(atts[i], _PT("nid"))) {
 			int dummyNid;
-			if (!SpiffReader::extractInteger(atts[1], 0, &dummyNid)) {
-				setError(SPIFF_READER_ERROR_ATTRIBUTE_INVALID,
+			if (!Toolbox::extractInteger(atts[i + 1], 0, &dummyNid)) {
+				handleError(SPIFF_READER_ERROR_ATTRIBUTE_INVALID,
 						SPIFF_READER_TEXT_ZERO_WRONG_ATTRIBUTE_TYPE(_PT("nid"), _PT("unsigned integer")));
 				return false;
 			}
 			this->d->extension.setNodeId(dummyNid);
 			nidFound = true;
+		} else if (SpiffReader::isXmlBase(atts[i])) {
+			// xml:base
+			XML_Char const * const xmlBase = atts[i + 1];
+			if (!handleXmlBaseAttribute(xmlBase)) {
+				return false;
+			}
 		} else {
-			setError(SPIFF_READER_ERROR_ATTRIBUTE_FORBIDDEN,
-					SPIFF_READER_TEXT_ONE_ATTRIBUTE_FORBIDDEN, atts[0]);
+			handleError(SPIFF_READER_ERROR_ATTRIBUTE_FORBIDDEN,
+					SPIFF_READER_TEXT_ONE_ATTRIBUTE_FORBIDDEN, atts[i]);
 			return false;
 		}
-		atts += 2;
 	}
 
 	if (!typeFound) {
-		setError(SPIFF_READER_ERROR_ATTRIBUTE_MISSING,
+		handleError(SPIFF_READER_ERROR_ATTRIBUTE_MISSING,
 				SPIFF_READER_TEXT_ZERO_ATTRIBUTE_MISSING(_PT("type")));
 		return false;
 	}
 
 	if (!nidFound) {
-		setError(SPIFF_READER_ERROR_ATTRIBUTE_MISSING,
+		handleError(SPIFF_READER_ERROR_ATTRIBUTE_MISSING,
 				SPIFF_READER_TEXT_ZERO_ATTRIBUTE_MISSING(_PT("nid")));
 		return false;
 	}
@@ -268,6 +268,5 @@ bool ProjectOpusPlaylistExtensionReader::handleInfoAttribs(const XML_Char ** att
 }
 
 
-
 }
 }
