@@ -1,7 +1,7 @@
 /*
  * libSpiff - XSPF playlist handling library
  *
- * Copyright (C) 2007, Sebastian Pipping / Xiph.Org Foundation
+ * Copyright (C) 2006-2008, Sebastian Pipping / Xiph.Org Foundation
  * All rights reserved.
  *
  * Redistribution  and use in source and binary forms, with or without
@@ -43,20 +43,19 @@
  * Implementation of SpiffPropsWriter.
  */
 
-#include <cstdlib>
-#include <spiff/SpiffPropsWriter.h>
+#include "SpiffPropsWriter.h"
 #include <spiff/SpiffProps.h>
+#include <spiff/SpiffDateTime.h>
 #include <spiff/SpiffXmlFormatter.h>
 #include <spiff/SpiffExtension.h>
 #include <spiff/SpiffExtensionWriter.h>
 #include <spiff/SpiffToolbox.h>
-using namespace std;
-using namespace Spiff::Toolbox;
+#include <list>
+#include <cstdlib>
 
 namespace Spiff {
 
 
-
 /// @cond DOXYGEN_NON_API
 
 /**
@@ -66,29 +65,25 @@ class SpiffPropsWriterPrivate {
 
 	friend class SpiffPropsWriter;
 
-	SpiffProps * props; ///< Playlist properties to write
-	int version; ///< XSPF version
+	SpiffProps props; ///< Playlist properties to write
 	bool trackListEmpty; ///< Tracklist empty flag, no initialization needed
-	std::list<std::pair<const XML_Char *, XML_Char *> > initNamespaces; ///< Namespaces to register when writing
+	std::list<std::pair<XML_Char const *, XML_Char *> > initNamespaces; ///< Namespaces to register when writing
+	bool embedBase; ///< Embed Base URI as xml:base flag, no initialization needed
 
 	/**
 	 * Creates a new D object.
 	 */
-	SpiffPropsWriterPrivate(SpiffProps * props)
-			: props(props),
-			version(-1) {
-
-	}
+	SpiffPropsWriterPrivate() { }
 
 	/**
 	 * Copy constructor.
 	 *
 	 * @param source  Source to copy from
 	 */
-	SpiffPropsWriterPrivate(const SpiffPropsWriterPrivate & source)
+	SpiffPropsWriterPrivate(SpiffPropsWriterPrivate const & source)
 			: props(source.props),
-			version(source.version),
-			trackListEmpty(source.trackListEmpty) {
+			trackListEmpty(source.trackListEmpty),
+			embedBase(source.embedBase) {
 		copyNamespaceInits(this->initNamespaces, source.initNamespaces);
 	}
 
@@ -97,11 +92,11 @@ class SpiffPropsWriterPrivate {
 	 *
 	 * @param source  Source to copy from
 	 */
-	SpiffPropsWriterPrivate & operator=(const SpiffPropsWriterPrivate & source) {
+	SpiffPropsWriterPrivate & operator=(SpiffPropsWriterPrivate const & source) {
 		if (this != &source) {
 			this->props = source.props;
-			this->version = source.version;
 			this->trackListEmpty = source.trackListEmpty;
+			this->embedBase = source.embedBase;
 			freeNamespaceInits(this->initNamespaces);
 			copyNamespaceInits(this->initNamespaces, source.initNamespaces);
 		}
@@ -109,11 +104,11 @@ class SpiffPropsWriterPrivate {
 	}
 
 	static void freeNamespaceInits(std::list<std::pair<
-			const XML_Char *, XML_Char *> >	& container) {
-		list<pair<const XML_Char *, XML_Char *> >::iterator
+			XML_Char const *, XML_Char *> >	& container) {
+		std::list<std::pair<XML_Char const *, XML_Char *> >::iterator
 			iter = container.begin();
 		while (iter != container.end()) {
-			pair<const XML_Char *, XML_Char *> & entry = *iter;
+			std::pair<XML_Char const *, XML_Char *> & entry = *iter;
 			delete [] entry.second;
 			iter++;
 		}
@@ -121,19 +116,19 @@ class SpiffPropsWriterPrivate {
 	}
 
 	static void copyNamespaceInits(
-			std::list<std::pair<const XML_Char *, XML_Char *> >
+			std::list<std::pair<XML_Char const *, XML_Char *> >
 				& dest,
-			const std::list<std::pair<const XML_Char *, XML_Char *> >
+			const std::list<std::pair<XML_Char const *, XML_Char *> >
 				& source) {
-		list<pair<const XML_Char *, XML_Char *> >::const_iterator
+		std::list<std::pair<XML_Char const *, XML_Char *> >::const_iterator
 			iter = source.begin();
 		while (iter != source.end()) {
-			const pair<const XML_Char *, XML_Char *> & entry = *iter;
+			const std::pair<XML_Char const *, XML_Char *> & entry = *iter;
 
-			const XML_Char * const uri = entry.first;
-			XML_Char * const prefixSuggestion = newAndCopy(entry.second);
+			XML_Char const * const uri = entry.first;
+			XML_Char * const prefixSuggestion = Toolbox::newAndCopy(entry.second);
 			dest.push_back(
-					pair<const XML_Char *, XML_Char *>(
+					std::pair<XML_Char const *, XML_Char *>(
 					uri, prefixSuggestion));
 
 			iter++;
@@ -152,24 +147,21 @@ class SpiffPropsWriterPrivate {
 /// @endcond
 
 
-
-SpiffPropsWriter::SpiffPropsWriter(SpiffProps * props)
-		: SpiffDataWriter(props),
-		d(new SpiffPropsWriterPrivate(props)) {
+SpiffPropsWriter::SpiffPropsWriter()
+		: SpiffDataWriter(),
+		d(new SpiffPropsWriterPrivate()) {
 
 }
 
 
-
-SpiffPropsWriter::SpiffPropsWriter(const SpiffPropsWriter & source)
+SpiffPropsWriter::SpiffPropsWriter(SpiffPropsWriter const & source)
 		: SpiffDataWriter(source),
 		d(new SpiffPropsWriterPrivate(*(source.d))) {
 
 }
 
 
-
-SpiffPropsWriter & SpiffPropsWriter::operator=(const SpiffPropsWriter & source) {
+SpiffPropsWriter & SpiffPropsWriter::operator=(SpiffPropsWriter const & source) {
 	if (this != &source) {
 		SpiffDataWriter::operator=(source);
 		*(this->d) = *(source.d);
@@ -178,37 +170,45 @@ SpiffPropsWriter & SpiffPropsWriter::operator=(const SpiffPropsWriter & source)
 }
 
 
-
 SpiffPropsWriter::~SpiffPropsWriter() {
 	delete this->d;
 }
 
 
+void
+SpiffPropsWriter::setProps(SpiffProps const * props) {
+	this->d->props = (props == NULL)
+		? SpiffProps()
+		: *props;
+	setData(&this->d->props);
+}
+
 
 void SpiffPropsWriter::writeAttribution() {
 	int index = 0;
-	pair<bool, const XML_Char *> * entry = this->d->props->getAttribution(index++);
+	std::pair<bool, XML_Char const *> * entry = this->d->props.getAttribution(index++);
 	if (entry == NULL) {
 		return;
 	}
 
-	const XML_Char * atts[1] = {NULL};
-	this->output->writeHomeStart(_PT("attribution"), atts);
+	XML_Char const * atts[1] = {NULL};
+	this->getOutput()->writeHomeStart(_PT("attribution"), atts);
 	do {
+		XML_Char * const relUri = makeRelativeUri(entry->second);
 		writePrimitive(entry->first ? _PT("location")
-				: _PT("identifier"), entry->second);
+				: _PT("identifier"), relUri);
+		delete [] relUri;
 		delete entry; // since the pair was created for us
-		entry = this->d->props->getAttribution(index++);
+		entry = this->d->props.getAttribution(index++);
 	} while (entry != NULL);
-	this->output->writeHomeEnd(_PT("attribution"));
+	this->getOutput()->writeHomeEnd(_PT("attribution"));
 }
 
 
-
 void SpiffPropsWriter::writeDate() {
-	const SpiffDateTime * const dateTime = this->d->props->getDate();
+	SpiffDateTime const * const dateTime = this->d->props.getDate();
 	if (dateTime != NULL) {
-		const int charCount = 10 + 1 + 8 + 6 + 1;
+		int const charCount = 10 + 1 + 8 + 6 + 1;
 		XML_Char buffer[charCount];
 		::PORT_SNPRINTF(buffer, charCount, _PT("%04i-%02i-%02iT%02i:%02i:%02i%s%02i:%02i"),
 				dateTime->getYear(), dateTime->getMonth(), dateTime->getDay(), dateTime->getHour(),
@@ -220,51 +220,52 @@ void SpiffPropsWriter::writeDate() {
 }
 
 
-
 void SpiffPropsWriter::writeIdentifier() {
-	const XML_Char * const identifier = this->d->props->getIdentifier();
+	XML_Char const * const identifier = this->d->props.getIdentifier();
 	if (identifier != NULL) {
-		writePrimitive(_PT("identifier"), identifier);
+		XML_Char * const relUri = makeRelativeUri(identifier);
+		writePrimitive(_PT("identifier"), relUri);
+		delete [] relUri;
 	}
 }
 
 
-
 void SpiffPropsWriter::writeLicense() {
-	const XML_Char * const license = this->d->props->getLicense();
+	XML_Char const * const license = this->d->props.getLicense();
 	if (license != NULL) {
-		writePrimitive(_PT("license"), license);
+		XML_Char * const relUri = makeRelativeUri(license);
+		writePrimitive(_PT("license"), relUri);
+		delete [] relUri;
 	}
 }
 
 
-
 void SpiffPropsWriter::writeLocation() {
-	const XML_Char * const location = this->d->props->getLocation();
+	XML_Char const * const location = this->d->props.getLocation();
 	if (location != NULL) {
-		writePrimitive(_PT("location"), location);
+		XML_Char * const relUri = makeRelativeUri(location);
+		writePrimitive(_PT("location"), relUri);
+		delete [] relUri;
 	}
 }
 
 
-
 void SpiffPropsWriter::writePlaylistClose() {
-	this->output->writeHomeEnd(_PT("playlist"));
+	this->getOutput()->writeHomeEnd(_PT("playlist"));
 }
 
 
-
 void SpiffPropsWriter::writePlaylistOpen() {
 	// Make namespace registration list
-	const int count = static_cast<int>(this->d->initNamespaces.size());
-	list<pair<const XML_Char *, XML_Char *> >::iterator iter
+	int const count = static_cast<int>(this->d->initNamespaces.size());
+	std::list<std::pair<XML_Char const *, XML_Char *> >::iterator iter
 			= this->d->initNamespaces.begin();
-	const XML_Char ** nsRegs = new const XML_Char *[2 * (1 + count) + 1];
+	XML_Char const ** nsRegs = new const XML_Char *[2 * (1 + count) + 1];
 	nsRegs[0] = SpiffXmlFormatter::namespaceKey;
 	nsRegs[1] = _PT("");
 	int walk = 2;
 	while (iter != this->d->initNamespaces.end()) {
-		pair<const XML_Char *, XML_Char *> & entry = *iter;
+		std::pair<XML_Char const *, XML_Char *> & entry = *iter;
 		nsRegs[walk] = entry.first;
 		nsRegs[walk + 1] = entry.second;
 		walk += 2;
@@ -273,13 +274,22 @@ void SpiffPropsWriter::writePlaylistOpen() {
 	nsRegs[walk] = NULL;
 
 	// Make version
-	const int charCount = 16;
+	int const charCount = 16;
 	XML_Char versionText[charCount];
-	::PORT_SNPRINTF(versionText, charCount, _PT("%i"), this->d->version);
-	const XML_Char * atts[3] = {_PT("version"), versionText, NULL};
+	::PORT_SNPRINTF(versionText, charCount, _PT("%i"),
+			this->d->props.getVersion());
+	XML_Char const * atts[2 + 2 + 1] = {_PT("version"), versionText,
+			NULL, NULL, NULL}; // Space for xml:base
+
+	// Make xml:base
+	XML_Char const * const baseUri = getBaseUri();
+	if (this->d->embedBase && (baseUri != NULL)) {
+		atts[0 + 2] = _PT("xml:base");
+		atts[1 + 2] = baseUri;
+	}
 
 	// Write tag
-	this->output->writeStart(SpiffXmlFormatter::namespaceKey,
+	this->getOutput()->writeStart(SpiffXmlFormatter::namespaceKey,
 			_PT("playlist"), atts, nsRegs);
 
 	// Delete reg list
@@ -288,60 +298,46 @@ void SpiffPropsWriter::writePlaylistOpen() {
 }
 
 
-
 void SpiffPropsWriter::writeTrackListClose() {
 	if (this->d->trackListEmpty) {
 		// Whole tag was written already
 		return;
 	}
-	this->output->writeHomeEnd(_PT("trackList"));
+	this->getOutput()->writeHomeEnd(_PT("trackList"));
 }
 
 
-
 void SpiffPropsWriter::writeTrackListOpen() {
-	const XML_Char * atts[1] = {NULL};
+	XML_Char const * atts[1] = {NULL};
 	if (this->d->trackListEmpty) {
-		if (this->d->version > 0) {
+		if (this->d->props.getVersion() > 0) {
 			// Empty trackList allowed
-			this->output->writeHomeStart(_PT("trackList"), atts);
-			this->output->writeHomeEnd(_PT("trackList"));
+			this->getOutput()->writeHomeStart(_PT("trackList"), atts);
+			this->getOutput()->writeHomeEnd(_PT("trackList"));
 		} else {
 			// Empty trackList forbidden
-			this->output->writeHomeStart(_PT("trackList"), atts);
-			this->output->writeHomeStart(_PT("track"), atts);
-			this->output->writeHomeEnd(_PT("track"));
-			this->output->writeHomeEnd(_PT("trackList"));
+			this->getOutput()->writeHomeStart(_PT("trackList"), atts);
+			this->getOutput()->writeHomeStart(_PT("track"), atts);
+			this->getOutput()->writeHomeEnd(_PT("track"));
+			this->getOutput()->writeHomeEnd(_PT("trackList"));
 		}
 	} else {
-		this->output->writeHomeStart(_PT("trackList"), atts);
+		this->getOutput()->writeHomeStart(_PT("trackList"), atts);
 	}
 }
 
 
-
-/**
- * Initializes the writer.
- *
- * @param output	Output formatter
- * @param version	XSPF version
- */
-void SpiffPropsWriter::init(SpiffXmlFormatter & output, int version) {
-	this->output = &output;
-	if ((version < 0) || (version > 1)) {
-		this->d->version = 1;
-	} else {
-		this->d->version = version;
-	}
+void
+SpiffPropsWriter::init(SpiffXmlFormatter & output,
+		XML_Char const * baseUri, bool embedBase) {
+	this->getOutput() = &output;
+	setBaseUri(baseUri);
+	this->d->embedBase = embedBase;
 }
 
 
-
-/**
- * Opens the <i>playlist</i> tag and writes all playlist
- * properties not handled by writeStartTracklist().
- */
-void SpiffPropsWriter::writeStartPlaylist() {
+void
+SpiffPropsWriter::writeStartPlaylist() {
 	writePlaylistOpen();
 	writeTitle();
 	writeCreator();
@@ -355,51 +351,39 @@ void SpiffPropsWriter::writeStartPlaylist() {
 	writeAttribution();
 	writeLinks();
 	writeMetas();
-	if (this->d->version > 0) {
+	if (this->d->props.getVersion() > 0) {
 		writeExtensions();
 	}
 }
 
 
-
-/**
- * Writes all playlist properties not handled by
- * writeStartPlaylist() and opens the <i>trackList</i> tag.
- *
- * @param trackListEmpty	Tracklist empty flag
- */
-void SpiffPropsWriter::writeStartTracklist(bool trackListEmpty) {
+void
+SpiffPropsWriter::writeStartTracklist(bool trackListEmpty) {
 	this->d->trackListEmpty = trackListEmpty;
 	writeTrackListOpen();
 }
 
 
-
-/**
- * Closes the <i>trackList</i> tag.
- */
-void SpiffPropsWriter::writeEndTracklist() {
+void
+SpiffPropsWriter::writeEndTracklist() {
 	writeTrackListClose();
 }
 
 
-
-/**
- * Closes the <i>playlist</i> tag.
- */
-void SpiffPropsWriter::writeEndPlaylist() {
+void
+SpiffPropsWriter::writeEndPlaylist() {
 	writePlaylistClose();
 }
 
 
-
-void SpiffPropsWriter::registerNamespace(const XML_Char * uri,
-		const XML_Char * prefixSuggestion) {
+bool
+SpiffPropsWriter::registerNamespace(XML_Char const * uri,
+		XML_Char const * prefixSuggestion) {
 	// TODO too late?
-	this->d->initNamespaces.push_back(pair<const XML_Char *,
-			XML_Char *>(uri, newAndCopy(prefixSuggestion)));
+	this->d->initNamespaces.push_back(std::pair<XML_Char const *,
+			XML_Char *>(uri, Toolbox::newAndCopy(prefixSuggestion)));
+	return true;
 }
 
 
-
 }
