@@ -1,7 +1,8 @@
 /*
  * libSpiff - XSPF playlist handling library
  *
- * Copyright (C) 2006 Sebastian Pipping
+ * Copyright (C) 2006-2008, Sebastian Pipping / Xiph.Org Foundation
+ * All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,31 +18,28 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * Sebastian Pipping, webmaster@hartwork.org
+ * Sebastian Pipping, sping@xiph.org
  */
 
 #ifndef SPIFF_TEST_EXTENSION_READER_FACTORY_SUITE_H
 #define SPIFF_TEST_EXTENSION_READER_FACTORY_SUITE_H 1
 
 
-
 #include "CppTest/cpptest.h"
 #include <spiff/SpiffDefines.h>
 
 
-
 class ExtensionReaderFactorySuite : public Test::Suite {
 
 public:
 	ExtensionReaderFactorySuite();
 
 private:
-	void test_register_helper(const XML_Char * testUri);
+	void test_register_helper(XML_Char const * testUri);
 	void test_register_uri();
 	void test_register_catch_all();
 
 };
 
 
-
 #endif // SPIFF_TEST_EXTENSION_READER_FACTORY_SUITE_H
